/******************************************************************************
*
* project name:    TI-Chess
* file name:       infoboards.c
* initial date:    12/07/2000
* authors:         thomas.nussbaumer@gmx.net (coding)
*                  marcos.lopez@wol.es (design/graphics/beta testing)
* description:     contains data output routines for info boards
*
* $Id: infoboards.c,v 1.2 2000/08/12 15:31:12 Thomas Nussbaumer Exp $
*
*******************************************************************************/

#if TI89_VERSION

//-----------------------------------------
// definition of infoboard positions (TI89)
//-----------------------------------------
#define IBOARD_START_X    104
#define IBOARD_START_Y0    57
#define IBOARD_START_Y1    12

#else

//-----------------------------------------
// definition of infoboard positions (TI92)
//-----------------------------------------
#define IBOARD_START_X    154
#define IBOARD_START_Y0    65
#define IBOARD_START_Y1    20

#endif



#define INFOBOARD_WIDTH  53
#define INFOBOARD_HEIGHT 30
#define INFO_OFFSET      23

#define REAL_PLAYER      -1



//----------------------------------
// structure describing an infoboard
//----------------------------------
typedef struct {
    int  x;            // x position of infoboard
    int  y;            // y position of infoboard
    int  player;       // type of player information
    char nodes[10];    // node counter information
    char move[20];     // best/last move
    char special1[25]; // special text1
    char special2[25]; // special text2
} infoboard_t;



//-------------------------------
// default settings of infoboards
//-------------------------------
infoboard_t infos[2] = {
    {IBOARD_START_X,IBOARD_START_Y0,-1, "", "",""}, // white
    {IBOARD_START_X,IBOARD_START_Y1, 0, "", "",""}  // black
};



//------------------------------------
// name lookup for calculator strength
//------------------------------------
char* levelnames[MAX_USERLEVEL] = {
 "Calc-L1","Calc-L2","Calc-L3","Calc-L4","Calc-L5"
};

int active_board = 0;   // the active board



//-------------------------------------------
// some forward declarations from other files
//-------------------------------------------
extern int already_mate;
char* Move2Str(move_t *pmove,int fromboard);



/*===========================================================================*/
/* draws info board                                                          */
/*===========================================================================*/
void DrawInfoBoard(int nr) {
    int x,y;

    x = infos[nr].x;
    y = infos[nr].y;

    //-----------------------------------------------------------------
    // fills background of infoboard with lightgray and draws separator
    //-----------------------------------------------------------------
    DrawColorRect(x,y,x+INFOBOARD_WIDTH,y+INFOBOARD_HEIGHT,COLOR_LIGHTGRAY,RECT_FILLED);
    SetPlane(LIGHT_PLANE);
    DrawLine(x,y-1,x+INFOBOARD_WIDTH,y-1,A_NORMAL);                                    // top
    DrawLine(x,y+INFOBOARD_HEIGHT+1,x+INFOBOARD_WIDTH,y+INFOBOARD_HEIGHT+1,A_NORMAL);  // bottom
    DrawLine(x-1,y,x-1,y+INFOBOARD_HEIGHT,A_NORMAL);                                   // left
    DrawLine(x+INFOBOARD_WIDTH+1,y,x+INFOBOARD_WIDTH+1,y+INFOBOARD_HEIGHT,A_NORMAL);   // right
    SetPlane(DARK_PLANE);
    DrawPix(x-1,y+5,A_NORMAL);
    DrawPix(x+INFOBOARD_WIDTH+1,y+5,A_NORMAL);
    DrawLine(x,y+6,x+INFOBOARD_WIDTH,y+6,A_NORMAL);


    //--------------------------------------------
    // output information depending on player type
    //--------------------------------------------
    FontSetSys(F_4x6);
    DrawStr(x,y,"Player:",A_NORMAL);
    if (infos[nr].player == REAL_PLAYER) {
       DrawStr(x+INFO_OFFSET+3,y,"Human",A_NORMAL);
       if (infos[nr].move[0] != 0) {
           DrawStr(x,y+8,"Last:",A_NORMAL);
           DrawStr(x+INFO_OFFSET,y+8,infos[nr].move,A_NORMAL);
       }
    }
    else {
       DrawStr(x+INFO_OFFSET+3,y,levelnames[infos[nr].player],A_NORMAL);

       if (infos[nr].move[0] != 0) {
           if (nr==active_board) DrawStr(x,y+8,"Best:",A_NORMAL);
           else                  DrawStr(x,y+8,"Last:",A_NORMAL);
           DrawStr(x+INFO_OFFSET,y+8,infos[nr].move,A_NORMAL);
       }
       if (infos[nr].nodes[0] != 0) {
           DrawStr(x,y+14,"Nodes:",A_NORMAL);
           DrawStr(x+INFO_OFFSET,y+14,infos[nr].nodes,A_NORMAL);
       }
    }
    DrawStr(x,y+20,infos[nr].special1,A_NORMAL);
    DrawStr(x,y+26,infos[nr].special2,A_NORMAL);
}



/*===========================================================================*/
/* update the "BEST MOVE SO FAR" display                                     */
/*===========================================================================*/
void UpdateMoveDisplay(move_t* pmove,int fromboard) {
    int x,y;

    x = infos[active_board].x;
    y = infos[active_board].y+8;

    //-------------------------------------------------------------------
    // fills background of movedisplay with lightgray and draws separator
    //-------------------------------------------------------------------
    DrawColorRect(x,y,x+INFOBOARD_WIDTH+1,y+5,COLOR_LIGHTGRAY,RECT_FILLED);

    sprintf(&(infos[active_board].move[0]),"%s",Move2Str(pmove,fromboard));

    SetPlane(DARK_PLANE);
    FontSetSys(F_4x6);
    DrawStr(x,y,"Best:",A_NORMAL);
    DrawStr(x+INFO_OFFSET,y,infos[active_board].move,A_NORMAL);
}



/*===========================================================================*/
/* updates node information                                                  */
/*===========================================================================*/
void UpdateNodeDisplay(long nodes) {
    int x,y;

    x = infos[active_board].x;
    y = infos[active_board].y+14;

    //-------------------------------------------------------------------
    // fills background of nodedisplay with lightgray and draws separator
    //-------------------------------------------------------------------
    DrawColorRect(x,y,x+INFOBOARD_WIDTH+1,y+5,COLOR_LIGHTGRAY,RECT_FILLED);

    sprintf(&(infos[active_board].nodes[0]),"%06ld",nodes);

    SetPlane(DARK_PLANE);
    FontSetSys(F_4x6);
    DrawStr(x,y,"Nodes:",A_NORMAL);
    DrawStr(x+INFO_OFFSET,y,infos[active_board].nodes,A_NORMAL);
}



/*===========================================================================*/
/* sets data of active infoboard                                             */
/*===========================================================================*/
void SetActiveInfo(int color,int level,int redraw) {
    if (infos[0].player == REAL_PLAYER) infos[0].nodes[0] = 0;
    if (infos[1].player == REAL_PLAYER) infos[1].nodes[0] = 0;


    if (color == WHITE) active_board = 0;
    else                active_board = 1;

    if (level>=0) {
        infos[active_board].player = level-1;
    }
    else {
        infos[active_board].player   = REAL_PLAYER;
        infos[active_board].nodes[0] = 0;
    }

    if (!already_mate) {
        infos[active_board].special1[0] = 0;
        infos[active_board].special2[0] = 0;
    }

    if (redraw) {
        DrawInfoBoard(0);
        DrawInfoBoard(1);
    }
}



/*===========================================================================*/
/* output special text on active infoboard                                   */
/*===========================================================================*/
void OutputSpecial(char* s1,char* s2) {
    int x,y;

    x = infos[active_board].x;
    y = infos[active_board].y+20;

    if (s1) sprintf(&(infos[active_board].special1[0]),"%s",s1);
    if (s2) sprintf(&(infos[active_board].special2[0]),"%s",s2);

    //-------------------------------------------------------------------
    // fills background of movedisplay with lightgray and draws separator
    //-------------------------------------------------------------------
    DrawColorRect(x,y,x+INFOBOARD_WIDTH+1,y+5,COLOR_LIGHTGRAY,RECT_FILLED);
    SetPlane(DARK_PLANE);
    FontSetSys(F_4x6);
    DrawStr(x,y,infos[active_board].special1,A_NORMAL);
    y+=6;
    DrawColorRect(x,y,x+INFOBOARD_WIDTH,y+5,COLOR_LIGHTGRAY,RECT_FILLED);
    SetPlane(DARK_PLANE);
    DrawStr(x,y,infos[active_board].special2,A_NORMAL);
}



/*===========================================================================*/
/* switches positions of infoboards                                          */
/*===========================================================================*/
void SwitchInfoBoards() {
    int y = infos[0].y;

    infos[0].y = infos[1].y;
    infos[1].y = y;
    DrawInfoBoard(0);
    DrawInfoBoard(1);
}



/*===========================================================================*/
/* clears active infobard data                                               */
/*===========================================================================*/
void ClearActiveInfoBoardData() {
    infos[active_board].nodes[0]    = 0;
    infos[active_board].move[0]     = 0;
    infos[active_board].special1[0] = 0;
    infos[active_board].special2[0] = 0;
}



/*===========================================================================*/
/* clears data of infoboard                                                  */
/*===========================================================================*/
void ClearInfoBoardData() {
    int nr;

    for (nr=0;nr<2;nr++) {
        infos[nr].nodes[0]    = 0;
        infos[nr].move[0]     = 0;
        infos[nr].special1[0] = 0;
        infos[nr].special2[0] = 0;
    }
}



/*===========================================================================*/
/* initialize both infoboards                                                */
/*===========================================================================*/
void InitInfo(int color,int level1,int level2,int inverted) {
    if (infos[0].player == REAL_PLAYER) infos[0].nodes[0] = 0;
    if (infos[1].player == REAL_PLAYER) infos[1].nodes[0] = 0;

    if (color == WHITE) active_board = 0;
    else                active_board = 1;

    if (level1>=0) infos[0].player = level1-1;
    else {
        infos[0].player = REAL_PLAYER;
        infos[0].nodes[0] = 0;
    }

    if (level2>=0) infos[1].player = level2-1;
    else {
        infos[1].player = REAL_PLAYER;
        infos[1].nodes[0] = 0;
    }


    if (!already_mate) {
        infos[0].special1[0] = 0;
        infos[0].special2[0] = 0;
        infos[1].special1[0] = 0;
        infos[1].special2[0] = 0;
    }

    if (infos[inverted].y != IBOARD_START_Y0) {
        SwitchInfoBoards();
    }
    else {
        DrawInfoBoard(0);
        DrawInfoBoard(1);
    }
}


//#############################################################################
//###################### NO MORE FAKES BEYOND THIS LINE #######################
//#############################################################################
//
//=============================================================================
// Revision History
//=============================================================================
//
// $Log: infoboards.c,v $
// Revision 1.2  2000/08/12 15:31:12  Thomas Nussbaumer
// substitution keywords added
//
//
 