/******************************************************************************
*
* project name:    TI-Chess
* file name:       interrupt.c
* initial date:    12/07/2000
* authors:         thomas.nussbaumer@gmx.net (coding)
*                  marcos.lopez@wol.es (design/graphics/beta testing)
* description:     keyboard handler routine installed as int1 handler
*
* !!! Thanx to Zeljko Juric to help me out with informations !!!
*
* $Id: interrupt.c,v 1.4 2000/12/09 15:36:38 Thomas Nussbaumer Exp $
*
******************************************************************************/


//-----------------------------------------------------------------------------
// One major problem is that the keyboard of the TI calculators is NOT
// unbounced! So don't believe in short transitions !!
//
// NOTE: Also the settle time of the register inquiry for _rowread() MUST be
//       increased by a factor of 3 for HW 2 calculators
//-----------------------------------------------------------------------------

#include "interrupt.h"

volatile unsigned char onkey     = 0;  // may used to detect [ON] key pressed for break
volatile unsigned int  do_reset  = 0;  // may be used to clear transition buffer
volatile unsigned char o_onkey   = 0;  // previous on key state
volatile unsigned char n_onkey   = 0;  // new on key state
volatile unsigned int  row2scan  = 0;  // actual row to scan
unsigned char          trow[10]  = {}; // may be used to detect key-down transitions
unsigned char          orow[10]  = {}; // previous key states
unsigned char          nrow[10]  = {}; // actual key states

INT_HANDLER            old_int1_handler = NULL; // backup variable for old int1 handler
INT_HANDLER            old_int2_handler = NULL; // backup variable for old int2 handler

int                    is_hw1_calc = 0;  // will be set to 1 if calculator is HW1
                                         // by InstallInterruptHandlers()


//-----------------------------------------------------------------------------
// use higher settle time for register query to fix problems with 
// HW2 calculators (redefinition of TIGCC macro)
//-----------------------------------------------------------------------------
#undef _rowread
#define _rowread(x) ({register int __r;*(int*)0x600018=(int)(x); \
  asm volatile("move.w #24,%0;0:dbra %0,0b":"=g"(__r)); \
  ~(*(unsigned char*)0x60001b);})



/*===========================================================================*/
/* our keyboard new keyboard handler detects and stores key down transitions */
/* (used internally)                                                         */
/*                                                                           */
/* this handler uses a round robin-like method, which means that at each     */
/* call just one row is scanned to keep the time effort for each call small  */
/*===========================================================================*/
DEFINE_INT_HANDLER (KeyboardHandler) {
    if (do_reset) {
        memset(trow,0,10);
        do_reset = 0;
        onkey    = 0;
    }

    nrow[row2scan] = _rowread(~(1<<row2scan));
    trow[row2scan] |= (~orow[row2scan]) & nrow[row2scan];
    orow[row2scan] = nrow[row2scan];

    row2scan++;
    if (!TI89) {
       if (row2scan == 10) row2scan = 0;
    }
    else {
       if (row2scan == 7)  row2scan = 0;
    }

    //-------------------------------------
    // get state of [ON] key
    //-------------------------------------
    n_onkey = ~(*(volatile unsigned char*)0x60001a) & 0x2;

    if (!o_onkey && n_onkey) onkey = 1;
    o_onkey = n_onkey;
}



/*===========================================================================*/
/* installs new INT handlers                                                 */
/*                                                                           */
/* INT1 handler == own keyboard handler                                      */
/* INT2 handler == dummy handler to increase performance (doesn't matter)    */
/*===========================================================================*/
void InstallInterruptHandlers() {
    unsigned long hwpb, *rombase,result;

    rombase = (unsigned long *)((*(unsigned long *)0xC8) & 0x600000);
    hwpb = rombase[65];
    result = (hwpb - (unsigned long)rombase < 0x10000 &&
    *(unsigned short *)hwpb > 22 ? *(unsigned long *)(hwpb + 22) : 1);

    if (result == 1) is_hw1_calc = 1;
    else             is_hw1_calc = 0;

    //---------------------------------------
    // initialize globals of keyboard handler
    //---------------------------------------
    onkey    = 0;
    o_onkey  = 0;
    n_onkey  = 0;
    do_reset = 0;
    row2scan = 0;
    memset(orow,0,10);
    memset(nrow,0,10);
    memset(trow,0,10);
    //---------------------------------------
    // install handler for INT1 and INT2
    //---------------------------------------
    old_int1_handler = GetIntVec(AUTO_INT_1);
    old_int2_handler = GetIntVec(AUTO_INT_2);
    SetIntVec(AUTO_INT_1,KeyboardHandler);
    SetIntVec(AUTO_INT_2,DUMMY_HANDLER);
}



/*===========================================================================*/
/* restores old INT handlers                                                 */
/*===========================================================================*/
void RestoreInterruptHandlers() {
    SetIntVec(AUTO_INT_1,old_int1_handler);
    SetIntVec(AUTO_INT_2,old_int2_handler);
}



//#############################################################################
//###################### NO MORE FAKES BEYOND THIS LINE #######################
//#############################################################################
//
//=============================================================================
// Revision History
//=============================================================================
//
// $Log: interrupt.c,v $
// Revision 1.4  2000/12/09 15:36:38  Thomas Nussbaumer
// using now an volatile pointer to fetch on key state
//
// Revision 1.3  2000/12/02 15:20:24  Thomas Nussbaumer
// using now interrupt support routines from TIGCC Library 2.22
//
// Revision 1.2  2000/08/12 15:31:13  Thomas Nussbaumer
// substitution keywords added
//
//
